/*
 * Copyright (c) 2008, Koji Tsukada<tsuka@mobiquitous.com>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;

using Phidgets;
using Phidgets.Events;
           
namespace MobiServer.Devices.Phidget.Control
{
    public partial class RFID : Base
    {
        Phidgets.RFID phidgetRFID = null;
        CheckBox[] checkBoxOutputs = null;

        string[] DISCARD_IDS = { "0c00000000", "0800000000" };

        public RFID(Phidgets.Phidget phidget) 
            : base(phidget)
        {
            InitializeComponent();
            InitComponentArray();
            phidgetRFID = (Phidgets.RFID)phidget;
            LoadSetting();
        }

        void InitComponentArray()
        {
            checkBoxOutputs = new CheckBox[3];
            checkBoxOutputs[0] = checkBoxOut0;
            checkBoxOutputs[1] = checkBoxOut1;
            checkBoxOutputs[2] = checkBoxLED;
            return;
        }
  
        override public bool Start()
        {
            try
            {
                if (!phidgetRFID.Attached)
                {
                    PrintError("No RFID found...");
                    PrintType();
                    return false;
                }

                PrintType();
                phidgetRFID.Antenna = true;
                phidgetRFID.Tag += new TagEventHandler(OnRFIDChange);
                phidgetRFID.TagLost += new TagEventHandler(OnRFIDLost);
            }

            catch (Exception ex)
            {
                PrintError(ex.Message);
                return false;
            }
            return true;
        }


        void OnRFIDChange(object sender, TagEventArgs args)
        {
            
            foreach (string id in DISCARD_IDS)
            {
                if (args.Tag == id)
                    return;
            }

            if (args.Tag != textBoxRFID.Text)
            {
                PrintDebug("Tag:{0}", args.Tag);
                textBoxRFID.Text = args.Tag;

                Data.RFID rfid = new Data.RFID(SerialNumber, Data.States.In, args.Tag);

                CreateServerWriteEvent(rfid.ToArray());
            }

            Application.DoEvents();
            return;
        }

        void OnRFIDLost(object sender, TagEventArgs args)
        {
            PrintDebug("Tag:Removed");
            textBoxRFID.Text = "0";

            Data.RFID rfid = new Data.RFID(SerialNumber, Data.States.In, "0");
            CreateServerWriteEvent(rfid.ToArray());
        }

        private void checkBoxLED_CheckedChanged(object sender, EventArgs e)
        {
            if (phidgetRFID != null && phidgetRFID.Attached)
            {
                phidgetRFID.LED = ((CheckBox)sender).Checked;
            }
        }

        private void checkBoxOut0_CheckedChanged(object sender, EventArgs e)
        {
            if (phidgetRFID != null && phidgetRFID.Attached)
            {
                phidgetRFID.outputs[0] = ((CheckBox)sender).Checked;
            }
        }

        private void checkBoxOut1_CheckedChanged(object sender, EventArgs e)
        {
            if (phidgetRFID != null && phidgetRFID.Attached)
            {
                phidgetRFID.outputs[1] = ((CheckBox)sender).Checked;
            }
        }

        public override void OnServerReadEvent(object sender, MobiServer.ServerEventArgs e)
        {
            try
            {
                if (CheckDeviceType(e.Args))
                {
                    Data.RFID r = Data.RFID.Parse(e.Args);

                    if (r != null)
                    {
                        switch (r.State)
                        {
                            case Data.States.Out:
                                if (r.Port < checkBoxOutputs.Length)
                                {
                                    if (r.Data == 1)
                                        checkBoxOutputs[r.Port].Checked = true;
                                    else if (r.Data == 0)
                                        checkBoxOutputs[r.Port].Checked = false;
                                }
                                break;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                PrintError(ex.Message);
            }
        }

        #region Save/Load
        
        override public void SaveSetting()
        {
            StreamWriter fileStream = null;
            string file = GetSettingFile();

            try
            {
                fileStream = new StreamWriter(file, false);
                for (int i = 0; i < checkBoxOutputs.Length; i++)
                {
                    fileStream.Write("{0}\t", checkBoxOutputs[i].Checked);
                }
            }
            catch (IOException ex)
            {
                PrintError(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Saving " + Path.GetFileName(file));
            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        override public void LoadSetting()
        {
            StreamReader fileStream = null;
            string file = GetSettingFile();
            try
            {
                if (File.Exists(file))
                {

                    fileStream = new StreamReader(file);

                    string line = null;
                    string[] args = null;

                    line = fileStream.ReadLine();

                    args = line.Split("\t".ToCharArray());

                    for (int i = 0; i < checkBoxOutputs.Length; i++)
                    {
                        checkBoxOutputs[i].Checked = bool.Parse(args[i]);
                    }
                }
            }
            catch (IOException ex)
            {
                PrintMessage(ex.Message);
            }
            catch (Exception ex)
            {
                PrintError("Loading " + Path.GetFileName(file));

            }
            finally
            {
                if (fileStream != null)
                {
                    fileStream.Close();
                    fileStream = null;
                }
            }
        }

        #endregion
        
    }
}

